@@ -296,16 +296,6 @@ lookup_stock_compat (const char *id)
 	return NULL;
 }
 
-static GHashTable *pixbuf_cache = NULL;
-
-void
-bonobo_ui_image_cache_trash (void)
-{
-	if (pixbuf_cache)
-		g_hash_table_destroy (pixbuf_cache);
-	pixbuf_cache = NULL;
-}
-
 void
 bonobo_ui_image_set_pixbuf (GtkImage *image, GdkPixbuf *pixbuf)
 {
@@ -330,6 +320,21 @@ bonobo_ui_image_set_stock (GtkImage   *image,
 		gtk_image_set_from_stock (image, name, icon_size);
 }
 
+static GtkIconSize
+bonobo_ui_util_xml_get_icon_size (BonoboUINode *node,
+				  GtkIconSize   default_size)
+{
+	GtkIconSize  retval;
+	const char  *size_name;
+
+	retval = default_size;
+
+	if ((size_name = bonobo_ui_node_peek_attr (node, "icon_size")))
+		retval = gtk_icon_size_from_name (size_name);
+
+	return retval;
+}
+
 void
 bonobo_ui_util_xml_set_image (GtkImage     *image,
 			      BonoboUINode *node,
@@ -339,15 +344,18 @@ bonobo_ui_util_xml_set_image (GtkImage     *image,
 	char       *key;
 	const char *type, *text;
 	GdkPixbuf  *pixbuf = NULL;
+	static GHashTable *pixbuf_cache = NULL;
 
 	g_return_if_fail (node != NULL);
 
-	if (!(type = bonobo_ui_node_peek_attr (node, "pixtype")) && cmd_node &&
-	    !(type = bonobo_ui_node_peek_attr (cmd_node, "pixtype")))
-		return;
+	if ((type = bonobo_ui_node_peek_attr (node, "pixtype"))) {
+		text = bonobo_ui_node_peek_attr (node, "pixname");
+		icon_size = bonobo_ui_util_xml_get_icon_size (node, icon_size);
 
-	if (!(text = bonobo_ui_node_peek_attr (node, "pixname")) && cmd_node &&
-	    !(text = bonobo_ui_node_peek_attr (cmd_node, "pixname")))
+	} else if (cmd_node && (type = bonobo_ui_node_peek_attr (cmd_node, "pixtype"))) {
+		text = bonobo_ui_node_peek_attr (cmd_node, "pixname");
+		icon_size = bonobo_ui_util_xml_get_icon_size (cmd_node, icon_size);
+	} else
 		return;
 
 	if (!strcmp (type, "stock")) {
@@ -363,10 +371,7 @@ bonobo_ui_util_xml_set_image (GtkImage     *image,
 		return;
 	}
 
-	/* FIXME: how worthwhile is this cache */
-	key = g_strdup_printf (
-		"%s:%d:%d", text, icon_size,
-		gtk_widget_get_direction (GTK_WIDGET (image)));
+	key = g_strdup_printf ("%s:%d", text, icon_size);
 
 	if (!pixbuf_cache)
 		pixbuf_cache = g_hash_table_new_full (
